# -*- coding: utf-8 -*-
from socket import *
from threading import Thread
import time

from Util import *


# SNMP 통신 처리 클래스

class SnmpCommunicator:
    def __init__(self):
        self.ip = '127.0.0.1'
        self.port = 52690
        self.isrun = False
              
    # 스레드 함수. TCP 서버를 시작, 클라이언트에서 메시지를 수신하고 처리
    def start(self):
        server_sock = socket(AF_INET, SOCK_STREAM)
        server_sock.setsockopt(SOL_SOCKET, SO_REUSEADDR, 1)
        server_sock.bind((self.ip, self.port))
        server_sock.listen(5)
        self.isrun = True
        print "Start SnmpCommunicator server listen {}, {}".format(self.ip, self.port)

        while self.isrun:
            try:
                (client_sock, addr) = server_sock.accept() 
               
                read_data = client_sock.recv(512)
                if len(read_data) < 4:
                    print "ReadData Error: " + len(read_data) #debug
                    client_sock.close()
                    client_sock = None
                    continue
                
                read_data = read_data.decode("utf-8") # bytes를 string으로 변경
                self.process_data(client_sock, read_data)  # 데이터 처리
                
                client_sock.close() # 클라이언트 소켓 종료
                           
            except:
                print "Error: Accept or Receive"
                
        server_sock.close()
        print "SnmpCommunicator server closed"
     
    # 스레드 종료   
    def stop(self):
        self.isrun = False
        
    # data의 crc를 계산하고 붙여서 리턴, data = "$KR,DAT_A" 와 같은 형식임
    def append_crc(self, data, cmdnum='1'):
        text = data + ',' + cmdnum  # command number 붙이기
        crc = 0
        for i in text :
            crc = 0xffff & (crc >> 8) | 0xffff & (crc << 8)
            crc ^= ord(i)
            crc ^= 0xffff & ((crc & 0xff) >> 4)
            crc ^= 0xffff & (crc << 12)
            crc ^= 0xffff & ((crc & 0x00ff) << 5)
            
        text = text + "*%04X\r\n" % crc
        return text
    
    # snmp get 요청의 응답 데이터 얻기.
    # command: 요청 명령
    def response_data(self, command):
        from Project import pj 
        
        isok = True
        # 메모리 효율을 위해 text += 'A' 방법은 피함
        if command == "VCS_MODE":
            text = "GET,%s,%d" % (command, (1 if pj.Vcs.AntSel == 3 else 0))
        elif command == "ACTIVE_ANT":
            if pj.AntAutoSel == 1:
                text = "GET,%s,A" % command
            elif pj.AntAutoSel == 2:
                text = "GET,%s,B" % command
            else:
                text = "GET,%s,N" % command
        elif command == "VCS_NIMLOCK":
            text = "GET,%s,%d" % (command, pj.Vcs.Link)
        elif command == "SYNC_MODE":
            text = "GET,{},{}".format(command, pj.Sys.GetSyncMode())
            
        elif command == "IN_REF_FREQ":
            text = "GET,{},{}".format(command, pj.McuComm.Mcu.DecTxInt)
        elif command == "EXT_REF_FREQ":
            text = "GET,{},{}".format(command, pj.McuComm.Mcu.DecTxExt)
        elif command == "MODEM_REF_FREQ":
            text = "GET,{},{}".format(command, pj.McuComm.Mcu.DecTxModem)
        elif command == "ACTIVE_REF_FREQ":
            text = "GET,{},{}".format(command, pj.Vcs.RefFreqSel)
        elif command == "TX_REF_FREQ":
            text = "GET,{},{}".format(command, pj.McuComm.Mcu.DecTxRefFreq)
            
        elif command == "MODEM_LOCK":
            text = "GET,{},{}".format(command, pj.Vcs.Lock)
            
        elif command == "GPS_VALID":
            text = "GET,{},{}".format(command, (1 if pj.GpsMgr.Valid == 1 else 0))
        elif command == "LATITUDE":
            text = "GET,%s,%0.1f" % (command, pj.GpsMgr.Lat)
        elif command == "LONGITUDE":
            text = "GET,%s,%0.1f" % (command, pj.GpsMgr.Lon)
        elif command == "GPS_SOURCE":
            text = "GET,%s,%d" % (command, (pj.GpsMgr.UseRmcIdx if pj.GpsMgr.UseRmcIdx > pj.GpsMgr.UseGgaIdx else pj.GpsMgr.UseGgaIdx))
            
        elif command == "GYRO_VALID":
            text = "GET,{},{}".format(command, (1 if pj.GyroMgr.Valid > 0 else 0))
        elif command == "GYRO_SOURCE":
            text = "GET,%s,%d" % (command, pj.GyroMgr.UseIdx)
        elif command == "HEADING":
            text = "GET,%s,%0.1f" % (command, pj.GyroMgr.Heading)
            
        elif command == "VCS_TEMPER":
            text = "GET,%s,%0.1f" % (command, Util.GetTemperature())
                         
        elif command == "ANTA_RUN":
            text = "GET,%s,%d" % (command, int(pj.AntA.Status.GetWithToken('Driving')))
        elif command == "ANTA_OPERATING_STATUS":
            text = "GET,%s,%d" % (command, int(pj.AntA.Status.GetWithToken('DrivingStatus')))
        elif command == "ANTA_OPERATING_MODE":
            text = "GET,%s,%d" % (command, int(pj.AntA.Status.GetWithToken('DrivingMode')))
        
        elif command == "ANTA_SKEWMODE":
            text = "GET,%s,%d" % (command, int(pj.AntA.Status.GetWithToken('SkewMode')))
        elif command == "ANTA_SKEW":
            text = "GET,%s,%0.1f" % (command, float(pj.AntA.Status.GetWithToken('Skew')))
        
        elif command == "ANTA_ELEVATION":
            text = "GET,%s,%0.1f" % (command, float(pj.AntA.Status.GetWithToken('Elevation')))
        elif command == "ANTA_AZIMUTH":
            text = "GET,%s,%0.1f" % (command, float(pj.AntA.Status.GetWithToken('Azimuth')))
            
        elif command == "ANTA_NIMAGC":
            text = "GET,%s,%d" % (command, int(pj.AntA.Status.GetWithToken('NimAgc')))
        elif command == "ANTA_NIMQUALITY":
            text = "GET,%s,%d" % (command, int(pj.AntA.Status.GetWithToken('NimQuality')))
        elif command == "ANTA_RSSD":
            text = "GET,%s,%d" % (command, int(pj.AntA.Status.GetWithToken('Rssd')))
        elif command == "ANTA_RSSDCN":
            text = "GET,%s,%d" % (command, int(pj.AntA.Status.GetWithToken('RssdCn')))
            
        elif command == "ANTA_SYNC_OFFSET1":
            text = "GET,%s,%0.1f" % (command, float(pj.AntA.GetSyncOffset1()))
        elif command == "ANTA_SYNC_OFFSET2":
            text = "GET,%s,%0.1f" % (command, float(pj.AntA.GetSyncOffset2()))
            
        elif command == "ANTB_RUN": # Antenna B
            text = "GET,%s,%d" % (command, int(pj.AntB.Status.GetWithToken('Driving')))
        elif command == "ANTB_OPERATING_STATUS":
            text = "GET,%s,%d" % (command, int(pj.AntB.Status.GetWithToken('DrivingStatus')))
        elif command == "ANTB_OPERATING_MODE":
            text = "GET,%s,%d" % (command, int(pj.AntB.Status.GetWithToken('DrivingMode')))
        
        elif command == "ANTB_SKEWMODE":
            text = "GET,%s,%d" % (command, int(pj.AntB.Status.GetWithToken('SkewMode')))
        elif command == "ANTB_SKEW":
            text = "GET,%s,%0.1f" % (command, float(pj.AntB.Status.GetWithToken('Skew')))
        
        elif command == "ANTB_ELEVATION":
            text = "GET,%s,%0.1f" % (command, float(pj.AntB.Status.GetWithToken('Elevation')))
        elif command == "ANTB_AZIMUTH":
            text = "GET,%s,%0.1f" % (command, float(pj.AntB.Status.GetWithToken('Azimuth')))
            
        elif command == "ANTB_NIMAGC":
            text = "GET,%s,%d" % (command, int(pj.AntB.Status.GetWithToken('NimAgc')))
        elif command == "ANTB_NIMQUALITY":
            text = "GET,%s,%d" % (command, int(pj.AntB.Status.GetWithToken('NimQuality')))
        elif command == "ANTB_RSSD":
            text = "GET,%s,%d" % (command, int(pj.AntB.Status.GetWithToken('Rssd')))
        elif command == "ANTB_RSSDCN":
            text = "GET,%s,%d" % (command, int(pj.AntB.Status.GetWithToken('RssdCn')))
            
        elif command == "ANTB_SYNC_OFFSET1":
            text = "GET,%s,%0.1f" % (command, float(pj.AntB.GetSyncOffset1()))
        elif command == "ANTB_SYNC_OFFSET2":
            text = "GET,%s,%0.1f" % (command, float(pj.AntB.GetSyncOffset2()))
            
        else:
            isok = False
        
        if isok:
            return self.append_crc(text)  # crc 붙여서 리턴
        else:
            return ""
    
    # snmp set 명령어 처리
    def process_snmpset(self, command, arg):
        from Project import pj 
        
        if command == "ACTIVE_ANT":
            antsel = 0
            if arg == 'A':
                antsel = 1
            elif arg == 'B':
                antsel = 2
                       
            pj.Vcs.AntSel = antsel
            pj.AntAutoSel = pj.Vcs.AntSel if pj.Vcs.AntSel == 1 or pj.Vcs.AntSel == 2 else pj.AntAutoSel 
            pj.McuComm.SendAntSelect(antsel) # MCU로 전송
            pj.Sys.UpdateVcs() # json 파일로 저장
        
        elif command == "SYNC_MODE":
            pj.Vcs.SyncMode = int(arg)
            pj.Sys.UpdateVcs()
        
        elif command == "ACTIVE_REF_FREQ":
            pj.Vcs.RefFreqSel = int(arg)
            pj.McuComm.SendRefFreqSelect(arg)
            pj.Sys.UpdateVcs()
        
        # 사용안함-사용자 모드일 경우에만 헤딩값 입력 가능.
        # elif command == "HEADING" and pj.GyroMgr.UseIdx == 5:  
        #     pj.Sys.Dat['Gyro']['UserInput'] = float(arg)
        #     pj.GyroMgr.Heading = float(arg)
        #     pj.Sys.Save()
        # 사용안함   
        # elif command == "ANTA_RUN":
        #     text = "$KW,DAT_A,%s" % 'RUN' if int(arg) == 1 else 'HLT'
        #     text = self.append_crc(text, '6') # crc 붙이기
        #     if pj.AntAInterface == 1: #랜 연결
        #         pj.AntCommA.Send(text)
        #     elif pj.AntAInterface == 2: #시리얼 연결
        #         pj.AntSubComm.Send(text)
                
        elif command == "ANTA_OPERATING_MODE":
            if int(arg) == 1:
                text = "$KW,DAT_H,MP" 
            elif int(arg) == 2:
                text = "$KW,DAT_H,MS"
            elif int(arg) == 3:
                text = "$KW,DAT_H,AS"
            else:
                text = "$KW,DAT_H,SB"
                
            text = self.append_crc(text) # crc 붙이기
            if pj.AntAInterface == 1: #랜 연결
                pj.AntCommA.Send(text)
            elif pj.AntAInterface == 2: #시리얼 연결
                pj.AntSubComm.Send(text)
            
        elif command == "ANTA_SKEWMODE":
            if int(arg) == 1:
                text = "$KW,DAT_L,MANL"
            elif int(arg) == 2:
                text = "$KW,DAT_L,SETP"
            elif int(arg) == 3:
                text = "$KW,DAT_L,RSTP"
            else:
                text = "$KW,DAT_L,AUTO"
                
            text = self.append_crc(text) # crc 붙이기
            if pj.AntAInterface == 1: #랜 연결
                pj.AntCommA.Send(text)
            elif pj.AntAInterface == 2: #시리얼 연결
                pj.AntSubComm.Send(text)
        elif command == "ANTA_SKEW":
            text = "$KW,DAT_L,MANL,%0.2f" % float(arg)
            text = self.append_crc(text) # crc 붙이기
            if pj.AntAInterface == 1: #랜 연결
                pj.AntCommA.Send(text)
            elif pj.AntAInterface == 2: #시리얼 연결
                pj.AntSubComm.Send(text)
            
        elif command == "ANTA_ELEVATION":
            #text = "$KW,DAT_I,ELE,%0.2f" % float(arg)
            text = "$KW,DAT_I,%0.2f,%0.2f" % (float(arg), float(pj.AntA.Status.GetWithToken('Azimuth')))
            text = self.append_crc(text, '6') # crc 붙이기
            if pj.AntAInterface == 1: #랜 연결
                pj.AntCommA.Send(text)
            elif pj.AntAInterface == 2: #시리얼 연결
                pj.AntSubComm.Send(text)
                
        elif command == "ANTA_AZIMUTH":
            #text = "$KW,DAT_I,AZI,%0.2f" % float(arg)
            text = "$KW,DAT_I,%0.2f,%0.2f" % (float(pj.AntA.Status.GetWithToken('Elevation')), float(arg))
            text = self.append_crc(text, '6') # crc 붙이기
            if pj.AntAInterface == 1: #랜 연결
                pj.AntCommA.Send(text)
            elif pj.AntAInterface == 2: #시리얼 연결
                pj.AntSubComm.Send(text)
        
        # Antenna B      
        elif command == "ANTB_OPERATING_MODE":
            if int(arg) == 1:
                text = "$KW,DAT_H,MP" 
            elif int(arg) == 2:
                text = "$KW,DAT_H,MS"
            elif int(arg) == 3:
                text = "$KW,DAT_H,AS"
            else:
                text = "$KW,DAT_H,SB"
                
            text = self.append_crc(text) # crc 붙이기
            if pj.AntBInterface == 1: #랜 연결
                pj.AntCommB.Send(text)
            elif pj.AntBInterface == 2: #시리얼 연결, MCU를 통해 메시지 전달
                pj.McuComm.Send(text)
            
        elif command == "ANTB_SKEWMODE":
            if int(arg) == 1:
                text = "$KW,DAT_L,MANL"
            elif int(arg) == 2:
                text = "$KW,DAT_L,SETP"
            elif int(arg) == 3:
                text = "$KW,DAT_L,RSTP"
            else:
                text = "$KW,DAT_L,AUTO"
                
            text = self.append_crc(text) # crc 붙이기
            if pj.AntBInterface == 1: #랜 연결
                pj.AntCommB.Send(text)
            elif pj.AntBInterface == 2: #시리얼 연결, MCU를 통해 메시지 전달
                pj.McuComm.Send(text)
        elif command == "ANTB_SKEW":
            text = "$KW,DAT_L,MANL,%0.2f" % float(arg)
            text = self.append_crc(text) # crc 붙이기
            if pj.AntBInterface == 1: #랜 연결
                pj.AntCommB.Send(text)
            elif pj.AntBInterface == 2: #시리얼 연결, MCU를 통해 메시지 전달
                pj.McuComm.Send(text)
            
        elif command == "ANTB_ELEVATION":
            #text = "$KW,DAT_I,ELE,%0.2f" % float(arg)
            text = "$KW,DAT_I,%0.2f,%0.2f" % (float(arg), float(pj.AntA.Status.GetWithToken('Azimuth')))
            text = self.append_crc(text, '6') # crc 붙이기
            if pj.AntBInterface == 1: #랜 연결
                pj.AntCommB.Send(text)
            elif pj.AntBInterface == 2: #시리얼 연결, MCU를 통해 메시지 전달
                pj.McuComm.Send(text)
                
        elif command == "ANTB_AZIMUTH":
            #text = "$KW,DAT_I,AZI,%0.2f" % float(arg)
            text = "$KW,DAT_I,%0.2f,%0.2f" % (float(pj.AntA.Status.GetWithToken('Elevation')), float(arg))
            text = self.append_crc(text, '6') # crc 붙이기
            if pj.AntBInterface == 1: #랜 연결
                pj.AntCommB.Send(text)
            elif pj.AntBInterface == 2: #시리얼 연결, MCU를 통해 메시지 전달
                pj.McuComm.Send(text)
    
        
    # 수신 데이터 처리
    def process_data(self, client_sock, read_data):
        # 데이터 분리
        arr = read_data.split(',') 
        if not arr or len(arr) < 3:
            return 
                
        if arr[0] == "GET":
            send_buff = self.response_data(arr[1]) # 응답 데이터 얻기
            try:
                client_sock.sendall(send_buff.encode("utf-8")) # 응답 데이터 전송
            except Exception, e:
                print "Error: TCP Send: " + e
        elif arr[0] == "SET":
            self.process_snmpset(arr[1], arr[2])
               
        
        
        
        