import socket

# 기존 ACU의 Excutor 기능을 구현
# 참고: 파이썬 3.x 에서 socket의 send() 함수와 recv()함수에서 주고받는 데이터 형식이 무조건 바이트 형식으로 변경됨. "UTF-8" 생략
# print()를 사용하면 SNMP 응답에 표시되므로 디버깅 용도로 사용한 후 주석처리할 것

class TcpClient:
    def __init__(self, ip='127.0.0.1', port=52690):
        self.ip = ip
        self.port = port
        self.result = None # 응답 데이터 결과 저장
      
    def request(self, data):
        # CRC 붙여 패킷 생성
        send_buff = TcpClient.append_crc(data) 
        
        client_socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        client_socket.settimeout(3) # 3초
        try:
            client_socket.connect((self.ip, self.port))
            client_socket.send(send_buff.encode())  # 데이터 송신.
            
            read_data = client_socket.recv(512)  # 응답 데이터 수신
            if not read_data or len(read_data) < 7:
                #print("ReadData Error: " + len(read_data)) #debug
                return False
            
            read_data = read_data.decode() # bytes를 string으로 변경
            
            client_socket.close()  # 소켓 종료
            
            self.process_data(read_data)  # 데이터 처리
        except Exception as e:
            # print("Request Error: ", e) # snmp 결과값에 표시되므로 삭제
            client_socket.close()
            return False
        
        return True
    
    def get_result(self):
        return self.result
    
    # static 함수와 비슷한 기능. 
    # data의 crc를 계산하고 붙여서 리턴, data = "$KR,DAT_A" 와 같은 형식임
    def append_crc(data):
        # 13(=0xD)은 command number이며, ACU에서는 이 값을 줘야 제대로 수신됨. 지금은 의미 없음
        text = data + ',13'  
        crc = 0
        for i in text :
            crc = 0xffff & (crc >> 8) | 0xffff & (crc << 8)
            crc ^= ord(i)
            crc ^= 0xffff & ((crc & 0xff) >> 4)
            crc ^= 0xffff & (crc << 12)
            crc ^= 0xffff & ((crc & 0x00ff) << 5)
            
        text = text + "*%04X\r\n" % crc
        return text
    
    # 수신 데이터 처리
    def process_data(self, read_data):
        self.result = ""
                
        arr = read_data.split(',') # 데이터 분리
        if not arr or len(arr) < 3:
            return 
        
        try:
            if arr[0] == "GET":
                self.result = arr[2]
                #if arr[1] == "VCS_MODE":
                #    self.result = arr[2]

        except Exception as e:
            #print("Process Data Error: ", e)
            pass